#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "usart.h"
#include "lcd.h"
#include "config.h"

void pwm_init()
{
	DDRD |= _BV(PD4) | _BV(PD5);

	TCCR1A |= (1 << COM1A1) | (1 << COM1B1) | (3 << WGM10);
	TCCR1B |= (1 << WGM12) | (1 << CS10);

	OCR1A = 0x7F;
	OCR1B = 0x7F;
}

void pwm_increase(int pwm_type)
{
	// increase the top
	if (pwm_type == PWM_COLD) {
		OCR1A += INC;
		if (OCR1A >= 0xFF)
			OCR1A = 0;
	}
	else if (pwm_type == PWM_WARM) {
		OCR1B += INC;
		if (OCR1B >= 0xFF)
			OCR1B = 0;
	}
}

void pwm_decrease(int pwm_type)
{
	if (pwm_type == PWM_COLD) {
		OCR1A -= INC;
		if (OCR1A <= 0)
			OCR1A = 0xFF;
	}
	else if (pwm_type == PWM_WARM) {
		OCR1B -= INC;
		if (OCR1B <= 0)
			OCR1B = 0xFF;
	}

}
void error()
{
	while (1) {
		PORTD ^= _BV(PD7);
		_delay_ms(200);
	}
}

int parse_command(const char *command, const char *param)
{
	if (strncmp(command, COMMAND_WARM, WARM_LEN) == 0) {
		if (strncmp(param, PARAM_UP, UP_LEN) == 0) {
			return WARM_UP;
		} else if (strncmp(param, PARAM_DOWN, DOWN_LEN) == 0) {
			return WARM_DOWN;
		} else {
			goto error;
		}

	} else if (strncmp(command, COMMAND_COLD, COLD_LEN) == 0) {
		if (strncmp(param, PARAM_UP, UP_LEN) == 0) {
			return COLD_UP;
		} else if (strncmp(param, PARAM_DOWN, DOWN_LEN) == 0) {
			return COLD_DOWN;
		} else {
			goto error;
		}
	} else {
		goto error;
	}

error:
	LCD_printAt(0x00, "Undefined command");
	return -1;
}

/**
 * Receives an unformatted @text as input, and
 * splits it into 2 tokens separated by " \r\n\t",
 * ignoring everything after the first 2 tokens.
 */
static int parse_text(char *text, int n)
{
	const char *delims = " \r\n\t";
	char command[n];
	char param[n];
	int count;
	char *s;

	count = 0;
	do {
		s = strsep(&text, delims);
		if (s == NULL || count == 2)
			break;
		else if (strlen(s) == 0)
			continue;
		else {
			if (count == 0)
				strcpy(command, s);
			else if (count == 1)
				strcpy(param, s);
			count++;
		}
	} while (1);
	if (count < 2)
		return -1;
	else
		return parse_command(command, param);
}

void adjust_brightness(int type) {

	if (type == WARM_UP)
		pwm_increase(PWM_WARM);
	else if (type == WARM_DOWN)
		pwm_decrease(PWM_WARM);
	else if (type == COLD_UP)
		pwm_increase(PWM_COLD);
	else if (type == COLD_DOWN)
		pwm_decrease(PWM_COLD);
}

int main(void)
{
	char buffer[100];
	int  size;
	int  rc;

	USART0_init();
	LCD_init();
	pwm_init();

	for (;;) {
		size = USART0_recvline(buffer);
		if (size == 0)
			error();
		rc = parse_text(buffer, size);

		memset(buffer, 0, 100);
		sprintf(buffer, "type = %i", rc);
		LCD_printAt(0x00, buffer);

		adjust_brightness(rc);

		_delay_ms(100);
	}

	return 0;
}

