#include "usart.h"
#include <util/delay.h>

/*
 * Functie de initializare a controllerului USART
 */
void USART0_init()
{
	DDRD |= (1 << PD1);
	DDRD &= ~(1 << PD0);

	/* seteaza baud rate la 115200 */
    UBRR0H = 0;
    UBRR0L = 8;

    /* seteaza formatul frame-ului: 8n1 (n - no parity) */
    UCSR0C = (3<<UCSZ00);

    /* porneste transmitatorul */
    UCSR0B = (1<<TXEN0) | (1<<RXEN0);
}

/*
 * Functie ce transmite un caracter prin USART
 *
 * @param data - caracterul de transmis
 */
void USART0_transmit(char data)
{
    /* asteapta pana bufferul e gol */
    while(!(UCSR0A & (1<<UDRE0)));

    /* pune datele in buffer; transmisia va porni automat in urma scrierii */
    UDR0 = data;
	_delay_ms(10);
}

/*
 * Functie ce primeste un caracter prin USART
 *
 * @return - caracterul primit
 */
char USART0_receive()
{
    /* asteapta cat timp bufferul e gol */
    while(!(UCSR0A & (1<<RXC0)));

    /* returneaza datele din buffer */
    return UDR0;
}

/*
 * Functie ce transmite un sir de caractere prin USART
 *
 * @param data - sirul (terminat cu '\0') de transmis
 */
void USART0_print(char *data)
{
    while(*data != '\0')
        USART0_transmit(*data++);
}

/*
 * Functie ce transmite un caracter prin USART
 *
 * @param data - caracterul de transmis
 * @param stream - nefolosit
 *
 * @return - intotdeauna returneaza 0
 */
int USART0_printf(char data, FILE *stream)
{
    /* asteapta pana bufferul e gol */
    while(!(UCSR0A & (1<<UDRE0)));

    /* pune datele in buffer; transmisia va porni automat in urma scrierii */
    UDR0 = data;
    return 0;
}

/* Function blocks until receiving a '\n'-terminated string,
 * which it puts in line_buf. Returns number of chars read,
 * including terminated '\n' */
int USART0_recvline(char *line_buf)
{
	int size = 0;
	char c;

	do {
		c = USART0_receive();
		line_buf[size++] = c;
	} while (c != '\n');
	line_buf[size] = 0;
	return size;
}
